@@ -942,7 +942,7 @@ panel_widget_switch_move (PanelWidget *panel, AppletData *ad, int moveby)
 		while (ad->pos < finalpos) {
 			pos = panel_widget_get_right_switch_pos (panel, list);
 			if (pos + ad->min_cells > panel->size ||
-			    abs (pos - finalpos) > abs (ad->pos - finalpos))
+			    abs (pos - finalpos) >= abs (ad->pos - finalpos))
 				break;
 			panel_widget_switch_applet_right (panel, list);
 		}
@@ -954,7 +954,7 @@ panel_widget_switch_move (PanelWidget *panel, AppletData *ad, int moveby)
 	} else {
 		while (ad->pos > finalpos) {
 			pos = panel_widget_get_left_switch_pos (panel, list);
-			if (pos < 0 || abs (pos - finalpos) > abs (ad->pos - finalpos))
+			if (pos < 0 || abs (pos - finalpos) >= abs (ad->pos - finalpos))
 				break;
 			panel_widget_switch_applet_left (panel, list);
 		}
@@ -1186,6 +1186,26 @@ queue_resize_on_all_applets(PanelWidget *panel)
 	}
 }
 
+static void
+panel_widget_set_background_region (PanelWidget *panel)
+{
+	GtkWidget *widget;
+	int       origin_x = -1, origin_y = -1;
+
+	widget = GTK_WIDGET (panel);
+
+	if (!GTK_WIDGET_REALIZED (widget))
+		return;
+
+	gdk_window_get_origin (widget->window, &origin_x, &origin_y);
+
+	panel_background_change_region (
+		&panel->background, panel->orient,
+		origin_x, origin_y,
+		widget->allocation.width,
+		widget->allocation.height);
+}
+
 static void
 panel_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 {
@@ -1364,6 +1384,8 @@ panel_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 			       0, ad->applet);
 	}
 	g_slist_free(send_move);
+
+	panel_widget_set_background_region (panel);
 }
 
 gboolean
@@ -1394,6 +1416,12 @@ panel_widget_is_cursor(PanelWidget *panel, int overlap)
 	return FALSE;
 }
 
+static void
+emit_background_changed (PanelWidget *panel_widget)
+{
+	g_signal_emit (panel_widget, panel_widget_signals [BACK_CHANGE_SIGNAL], 0);
+}
+
 void
 panel_widget_set_back_pixmap (PanelWidget *panel,
 			      const char  *image_path)
@@ -1401,8 +1429,8 @@ panel_widget_set_back_pixmap (PanelWidget *panel,
 	g_return_if_fail (PANEL_IS_WIDGET (panel));
 	g_return_if_fail (image_path != NULL);
 
-	panel_background_set_image (
-		&panel->background, image_path, FALSE, FALSE, FALSE);
+	if (panel_background_set_image (&panel->background, image_path, FALSE, FALSE, FALSE))
+		emit_background_changed (panel);
 
 	gtk_widget_queue_draw (GTK_WIDGET (panel));
 }
@@ -1414,7 +1442,8 @@ panel_widget_set_back_color (PanelWidget *panel,
 	g_return_if_fail (PANEL_IS_WIDGET (panel));
 	g_return_if_fail (color != NULL);
 
-	panel_background_set_color (&panel->background, color);
+	if (panel_background_set_color (&panel->background, color))
+		emit_background_changed (panel);
 
 	gtk_widget_queue_draw (GTK_WIDGET (panel));
 }
@@ -1446,12 +1475,7 @@ toplevel_configure_event (GtkWidget         *widget,
 			  GdkEventConfigure *event,
 			  PanelWidget       *panel)
 {
-	panel_background_change_region (
-		&panel->background, panel->orient,
-		widget->allocation.x,
-		widget->allocation.y,
-		widget->allocation.width,
-		widget->allocation.height);
+	panel_widget_set_background_region (panel);
 
 	return FALSE;
 }
@@ -1607,6 +1631,8 @@ panel_widget_instance_init (PanelWidget *panel)
 	panel->master_widget = NULL;
 	panel->drop_widget   = widget;
 	panel->inhibit_draw  = FALSE;
+	panel->delete_dialog = NULL;
+	panel->create_launcher_dialog_list = NULL;
 
 	panel_background_init (&panel->background);
 
@@ -1614,12 +1640,12 @@ panel_widget_instance_init (PanelWidget *panel)
 }
 
 GtkWidget *
-panel_widget_new (char                *panel_id,
+panel_widget_new (const char          *panel_id,
 		  gboolean             packed,
 		  GtkOrientation       orient,
 		  int                  sz,
 		  PanelBackgroundType  back_type,
-		  char                *back_pixmap,
+		  const char          *back_pixmap,
 		  gboolean             fit_pixmap_bg,
 		  gboolean             stretch_pixmap_bg,
 		  gboolean             rotate_pixmap_bg,
@@ -1637,9 +1663,10 @@ panel_widget_new (char                *panel_id,
 	else
 		panel->unique_id = g_strdup (panel_id);
 
-	panel_background_set (
+	if (panel_background_set (
 		&panel->background, back_type, back_color, back_pixmap,
-		fit_pixmap_bg, stretch_pixmap_bg, rotate_pixmap_bg);
+		fit_pixmap_bg, stretch_pixmap_bg, rotate_pixmap_bg))
+		emit_background_changed (panel);
 
 	panel->orient = orient;
 	panel->sz = sz;
@@ -2410,6 +2437,7 @@ panel_widget_reparent (PanelWidget *old_panel,
 {
 	AppletData *ad;
 	GtkWidget *focus_widget = NULL;
+	AppletInfo* info;
 
 	g_return_val_if_fail(PANEL_IS_WIDGET(old_panel),-1);
 	g_return_val_if_fail(PANEL_IS_WIDGET(new_panel),-1);
@@ -2426,7 +2454,9 @@ panel_widget_reparent (PanelWidget *old_panel,
 	    (BASEP_WIDGET (new_panel->panel_parent)->state == BASEP_HIDDEN_LEFT ||
 	     BASEP_WIDGET (new_panel->panel_parent)->state == BASEP_HIDDEN_RIGHT))
 		return FALSE;
-	
+
+	info = g_object_get_data (G_OBJECT (ad->applet), "applet_info");
+
 	/*we'll resize both panels anyway*/
 	ad->dirty = FALSE;
 	
@@ -2440,6 +2470,9 @@ panel_widget_reparent (PanelWidget *old_panel,
 	}
 	gtk_widget_reparent (applet, GTK_WIDGET (new_panel));
 
+	if (info && info->type == APPLET_BONOBO)
+		panel_applet_frame_set_panel (PANEL_APPLET_FRAME (ad->applet), new_panel);
+
 	if (GTK_WIDGET_CAN_FOCUS (new_panel))
 		GTK_WIDGET_UNSET_FLAGS (new_panel, GTK_CAN_FOCUS);
 	if (focus_widget) {
@@ -2465,7 +2498,7 @@ panel_widget_change_params(PanelWidget         *panel,
 			   GtkOrientation       orient,
 			   int                  sz,
 			   PanelBackgroundType  back_type,
-			   char                *pixmap,
+			   const char          *pixmap,
 			   gboolean             fit_pixmap_bg,
 			   gboolean             stretch_pixmap_bg,
 			   gboolean             rotate_pixmap_bg,
@@ -2497,13 +2530,8 @@ panel_widget_change_params(PanelWidget         *panel,
 	change_back = panel_background_set (
 			&panel->background, back_type, back_color, pixmap,
 			fit_pixmap_bg, stretch_pixmap_bg, rotate_pixmap_bg);
-	if (change_back) {
-		panel_background_realized (
-			&panel->background, GTK_WIDGET (panel)->window);
-		
-		g_signal_emit (
-			panel, panel_widget_signals [BACK_CHANGE_SIGNAL], 0);
-	}
+	if (change_back)
+		emit_background_changed (panel);
 
 	/* inhibit draws until we resize */
 	panel->inhibit_draw = TRUE;
@@ -2682,3 +2710,26 @@ panel_widget_get_applet_orient (PanelWidget *panel)
 
 	return basep_widget_get_applet_orient (BASEP_WIDGET (panel->panel_parent));
 }
+
+void
+panel_widget_remove_drawers (PanelWidget *panel_widget)
+{
+	GList *l;
+
+	for (l = panel_widget->applet_list; l ; l = l->next) {
+
+		AppletData* applet_data = l->data;
+		AppletInfo* info = g_object_get_data (
+						G_OBJECT (applet_data->applet), "applet_info");
+
+		if (info && info->type == APPLET_DRAWER) {
+			Drawer *drawer = (Drawer *) info->data;
+			BasePWidget* drawer_widget = (BASEP_WIDGET (drawer->drawer));
+
+			panel_widget_remove_drawers (PANEL_WIDGET(drawer_widget->panel));
+
+			panel_remove_from_gconf(PANEL_WIDGET(drawer_widget->panel));
+		}
+	}
+
+}
